#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <float.h>
#include "iio.h"



typedef struct image_float_s
{
  float * data;
  unsigned int xsize,ysize;
} * image_float;


static void fatal_error(char * msg)
{
  fprintf(stderr,"Error: %s\n",msg);
  exit(EXIT_FAILURE);
}





/*----------------------------------------------------------------------------*/
/** Create a new image_float of size 'xsize' times 'ysize'.
 */
static image_float new_image_float(unsigned int xsize, unsigned int ysize)
{
  image_float image;

  /* check parameters */
  if( xsize == 0 || ysize == 0 ) fatal_error("new_image_float: invalid image size.");

  /* get memory */
  image = (image_float) malloc( sizeof(struct image_float_s) );
  if( image == NULL ) fatal_error("not enough memory.");
  image->data = (float *) calloc( (size_t) (xsize*ysize), sizeof(float) );
  if( image->data == NULL ) fatal_error("not enough memory.");

  /* set image size */
  image->xsize = xsize;
  image->ysize = ysize;

  return image;
}
float read_image_float(const char *fname, float **f, int *w, int *h)
{
        float *f_float;
	f_float = iio_read_image_float(fname, w, h);
        *f = (float *)calloc((*w * *h),(unsigned int)sizeof(float));   
        for(int i = 0; i < *w * *h; i++)
	   (*f)[i] =   f_float[i];

	return *f ? 0 : -1;
}




static void getminmax(float *min, float *max, const float *x, int n)
{
	*min = *max = x[0];
	for (int i = 1; i < n; i++) {
		if (x[i] < *min)
			*min = x[i];
		if (x[i] > *max)
			*max = x[i];
	}
}


/*----------------------------------------------------------------------------*/
/** Free memory used in image_float 'i'.
 */
static void free_image_float(image_float i)
{
  if( i == NULL || i->data == NULL )
    fatal_error("free_image_float: invalid input image.");
  free( (void *) i->data );
  free( (void *) i );
}




int main(int argc, char *argv[])
{
    if(argc < 2)
        printf("image out: ");
    else
    {
        //read the parameters
        int i=1;
        const char* image  = argv[i]; i++;
        char* outimage = argv[i]; i++;

	int nx, ny;
        image_float  imoutfloat;
	float min, max,*im;
      
        if((read_image_float(image, &im, &nx, &ny) != -1))
        {
            //allocate memory for the flow
            
	      getminmax(&min,&max, im, nx*ny);

  	     const float den = max - min; 	
                    
	    imoutfloat = new_image_float(nx,ny);
	    if(den!=0.) for ( i = 0; i < nx * ny; i++) 
              imoutfloat->data[i]= (float) round(255 * (im[i] - min) / den);
	   
	    iio_save_image_float(outimage,imoutfloat->data,imoutfloat->xsize,imoutfloat->ysize);

            free_image_float(imoutfloat);
    
      
        }
        else  fatal_error("Cannot read the images");
        
    }

    return 0;
}

