#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <float.h>
#include<string.h>
#include "iio.h"
#include "nonlocal_average.h"


static void fatal_error(char * msg)
{
  fprintf(stderr,"Error: %s\n",msg);
  exit(EXIT_FAILURE);
}

void nonlocal_average(int *label,int *ima,int nx,int ny,float *out)
{
float npix1 = 0, npix0 = 0, sum_one = 0, sum_zero = 0;
int i,j,Nlabels=0;
unsigned int *nPix;
int *vecLabels,found;
float *sum_values;
/* allocate memory for vecLabels*/
vecLabels = (int *) calloc( (unsigned int) (NMAXMABELS), sizeof(int)+1);
sum_values = (float *) calloc( (unsigned int) (NMAXMABELS), sizeof(float)+1);
nPix = (unsigned int *) calloc( (unsigned int) (NMAXMABELS), sizeof(unsigned int)+1);

/*how many labels there are? */
vecLabels[0] = label[0];
nPix[0] = 1;
for(i=0;i<nx*ny;i++){   
        found = 0;
	for(j=0;j<=Nlabels;j++){
		if(label[i]==vecLabels[j]){
			found = 1;
		}
	}
	if(found==0){
		Nlabels++;
		vecLabels[Nlabels]=label[i];
	}	
}

/* a vector of float that stores the averages of the image values over the same label */
for(i=0;i<nx*ny;i++){
	for(j=0;j<=Nlabels;j++){
		if(label[i]==vecLabels[j]){
 		nPix[j] +=1;
 		sum_values[j] += ima[i];
	}}
}

for(j=0;j<=Nlabels;j++){
	sum_values[j] = (float)sum_values[j]/(float)nPix[j];
}

for (i = 0; i < nx*ny; i++){
		for(j=0;j<=Nlabels;j++){
			if(label[i]==vecLabels[j]){
				out[i] = sum_values[j];
			}
		}
}

free((void *) vecLabels);
free((void *) nPix);

free((void *) sum_values);

}


int read_image_int(const char *fname, int **f, int *w, int *h)
{
        float *f_float;
    
	f_float = iio_read_image_float(fname, w, h);
        *f = (int *)calloc((*w * *h),(unsigned int)sizeof(int));   
        for(int i = 0; i < *w * *h; i++)
	   (*f)[i] = (int)  f_float[i];
        free((float *) f_float);
	return *f ? 0 : -1;
}



typedef struct image_float_s
{
  float * data;
  unsigned int xsize,ysize;
} * image_float;



/*----------------------------------------------------------------------------*/
/** Create a new image_float of size 'xsize' times 'ysize'.
 */
static image_float new_image_float(unsigned int xsize, unsigned int ysize)
{
  image_float image;

  /* check parameters */
  if( xsize == 0 || ysize == 0 ) fatal_error("new_image_float: invalid image size.");

  /* get memory */
  image = (image_float) malloc( sizeof(struct image_float_s) );
  if( image == NULL ) fatal_error("not enough memory.");
  image->data = (float *) calloc( (size_t) (xsize*ysize), sizeof(float) );
  if( image->data == NULL ) fatal_error("not enough memory.");

  /* set image size */
  image->xsize = xsize;
  image->ysize = ysize;

  return image;
}
 




int read_image_float(const char *fname, float **f, int *w, int *h)
{
        float *f_float;
	f_float = iio_read_image_float(fname, w, h);
        *f = (float *)calloc((*w * *h),(unsigned int)sizeof(float));   
        for(int i = 0; i < *w * *h; i++)
	   (*f)[i] = (float)  f_float[i];

	return *f ? 0 : -1;
}
static void free_image_float(image_float i)
{
  if( i == NULL || i->data == NULL )
    fatal_error("free_image_float: invalid input image.");
  free( (void *) i->data );
  free( (void *) i );
}

static void getminmax(float *min, float *max, const float *x, int n)
{
	*min = *max = x[0];
	for (int i = 1; i < n; i++) {
		if (x[i] < *min)
			*min = x[i];
		if (x[i] > *max)
			*max = x[i];
	}
}


void im_normalization(float  *im, int nx, int ny, image_float imoutfloat)
{
int i;
float min, max;

  getminmax(&min,&max, im, nx*ny);


  	     const float den = max - min; 	
                    
	    if(den!=0.) for ( i = 0; i < nx * ny; i++) 
              imoutfloat->data[i]= (float) round(255 * (im[i] - min) / den);

}

int main(int argc, char *argv[])
{
    if(argc < 3)
      {
        printf("image out: ");
     
      }
    else
    {
        /* Declare variables */
	int nx, ny;

        int *im1,*im2;
        char fname[100];
        float  *out;
        image_float imout;
        /* Read the parameters */
        int i = 1;
        const char* image1  = argv[i]; i++;//rough image
        const char* image2  = argv[i]; i++;//label image (original)
        const char* outfile = argv[i]; i++;
 

        if((read_image_int(image1, &im1, &nx, &ny) != -1) && (read_image_int(image2, &im2, &nx, &ny) != -1))
        {
            /* allocate memory for the flow */
            imout = new_image_float(nx,ny);
	    out = (float *) calloc( (unsigned int) (nx*ny), sizeof(float) );
	    /* initialise */

            nonlocal_average(im2, im1, nx, ny,out); /* */
            im_normalization(out, nx, ny, imout);
            sprintf(fname, "%s.pgm", outfile);
	    iio_save_image_float(fname,imout->data,imout->xsize,imout->ysize);


            free_image_float(imout);   
	    free(out);
 		free((int *)im1);
 		free((int *)im2);
        }
        else{
             fatal_error("Cannot read the images");
        }
    }
    return 0;
}


